# This file was auto-generated by Fern from our API Definition.

from .types import (
    Asset,
    CreateAssetDevice,
    CreateAssetRequest,
    CreateAssetResponse,
    CreateAssetStatus,
    CreateFindingsError,
    CreateFindingsRequest,
    CreateFindingsResponse,
    GetScanActivityResponse,
    GetScanStatusResponse,
    NucleusFindingState,
    ProviderSpecificFindingState,
    ProviderSpecificFindingState_Nucleus,
    QueryAssetsResponse,
    QueryFindingsResponse,
    QueryScansResponse,
    ScanConfiguration,
    ScanFrequencyOption,
    ScanSchedule,
    ScanStatus,
    SecurityFinding,
    UpdateFindingRequest,
    UploadScanRequest,
    UploadScanResponse,
    UploadScanStatus,
    User,
    VulnerabilitySeverityFilterValue,
    VulnerabilityStateFilterValue,
)

__all__ = [
    "Asset",
    "CreateAssetDevice",
    "CreateAssetRequest",
    "CreateAssetResponse",
    "CreateAssetStatus",
    "CreateFindingsError",
    "CreateFindingsRequest",
    "CreateFindingsResponse",
    "GetScanActivityResponse",
    "GetScanStatusResponse",
    "NucleusFindingState",
    "ProviderSpecificFindingState",
    "ProviderSpecificFindingState_Nucleus",
    "QueryAssetsResponse",
    "QueryFindingsResponse",
    "QueryScansResponse",
    "ScanConfiguration",
    "ScanFrequencyOption",
    "ScanSchedule",
    "ScanStatus",
    "SecurityFinding",
    "UpdateFindingRequest",
    "UploadScanRequest",
    "UploadScanResponse",
    "UploadScanStatus",
    "User",
    "VulnerabilitySeverityFilterValue",
    "VulnerabilityStateFilterValue",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class HttpHeader(UncheckedBaseModel):
    """
    TThe HTTP Header object represents the headers sent in an HTTP request or response. HTTP headers are key-value pairs that convey additional information about the HTTP message, including details about the content, caching, authentication, encoding, and other aspects of the communication.
    """

    name: str = pydantic.Field()
    """
    The name of the header
    """

    value: str = pydantic.Field()
    """
    The value of the header
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .pager_duty_credential import PagerDutyCredential
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TicketingPagerDuty(UncheckedBaseModel):
    """
    Configuration for PagerDuty Operations Cloud.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/pagerduty-ticketing-setup)
    """

    credential: PagerDutyCredential
    default_creator: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default email address for the creator field. If provided, the creator field becomes optional for ticket and comment creation. If not provided, the creator field is required.
    """

    default_project: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default PagerDuty service ID for ticket creation. If provided, the project field becomes optional in ticket creation requests. If not provided, the project field is required.
    """

    url: str = pydantic.Field()
    """
    Base URL for the PagerDuty API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from .types.list_storage_response import ListStorageResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from ..common.errors.not_implemented_error import NotImplementedError
from ..common.errors.bad_gateway_error import BadGatewayError
from ..common.errors.service_unavailable_error import ServiceUnavailableError
from ..common.errors.gateway_timeout_error import GatewayTimeoutError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .. import core
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class StorageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_files(
        self,
        path: str,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStorageResponse:
        """
        Returns a list of contents from the token-linked `Integration`.

        Parameters
        ----------
        path : str

        cursor : typing.Optional[str]
            Cursor to fetch the next set of results.

        limit : typing.Optional[int]
            Number of results to return. Default is 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListStorageResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.storage.list_files(
            path="string",
            cursor="string",
            limit=1,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/storage/folders/{jsonable_encoder(path)}",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListStorageResponse,
                    construct_type(
                        type_=ListStorageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upload_file(
        self,
        path: str,
        *,
        file: core.File,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Uploads a file from the provided `{path}` to the token-linked `Integration`.

        Parameters
        ----------
        path : str

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.storage.upload_file(
            path="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def download_file(
        self, path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Downloads a file from the provided `{path}` in the token-linked `Integration`.

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[bytes]

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.storage.download_file(
            path="string",
        )
        """
        with self._client_wrapper.httpx_client.stream(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="GET",
            request_options=request_options,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    for _chunk in _response.iter_bytes():
                        yield _chunk
                    return
                _response.read()
                if _response.status_code == 400:
                    raise BadRequestError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 401:
                    raise UnauthorizedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 403:
                    raise ForbiddenError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 404:
                    raise NotFoundError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 405:
                    raise MethodNotAllowedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 409:
                    raise ConflictError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 415:
                    raise UnsupportedMediaTypeError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 429:
                    raise TooManyRequestsError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 500:
                    raise InternalServerError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 501:
                    raise NotImplementedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 502:
                    raise BadGatewayError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 503:
                    raise ServiceUnavailableError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 504:
                    raise GatewayTimeoutError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_file(
        self, path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a file from the provided `{path}` in the token-linked `Integration`.

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.storage.delete_file(
            path="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncStorageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_files(
        self,
        path: str,
        *,
        cursor: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListStorageResponse:
        """
        Returns a list of contents from the token-linked `Integration`.

        Parameters
        ----------
        path : str

        cursor : typing.Optional[str]
            Cursor to fetch the next set of results.

        limit : typing.Optional[int]
            Number of results to return. Default is 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListStorageResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.storage.list_files(
                path="string",
                cursor="string",
                limit=1,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/storage/folders/{jsonable_encoder(path)}",
            method="GET",
            params={
                "cursor": cursor,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListStorageResponse,
                    construct_type(
                        type_=ListStorageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upload_file(
        self,
        path: str,
        *,
        file: core.File,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Uploads a file from the provided `{path}` to the token-linked `Integration`.

        Parameters
        ----------
        path : str

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.storage.upload_file(
                path="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="POST",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def download_file(
        self, path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Downloads a file from the provided `{path}` in the token-linked `Integration`.

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[bytes]

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.storage.download_file(
                path="string",
            )


        asyncio.run(main())
        """
        async with self._client_wrapper.httpx_client.stream(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="GET",
            request_options=request_options,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    async for _chunk in _response.aiter_bytes():
                        yield _chunk
                    return
                await _response.aread()
                if _response.status_code == 400:
                    raise BadRequestError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 401:
                    raise UnauthorizedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 403:
                    raise ForbiddenError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 404:
                    raise NotFoundError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 405:
                    raise MethodNotAllowedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 409:
                    raise ConflictError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 415:
                    raise UnsupportedMediaTypeError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 429:
                    raise TooManyRequestsError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 500:
                    raise InternalServerError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 501:
                    raise NotImplementedError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 502:
                    raise BadGatewayError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 503:
                    raise ServiceUnavailableError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                if _response.status_code == 504:
                    raise GatewayTimeoutError(
                        typing.cast(
                            Problem,
                            construct_type(
                                type_=Problem,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_file(
        self, path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Deletes a file from the provided `{path}` in the token-linked `Integration`.

        Parameters
        ----------
        path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.storage.delete_file(
                path="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/storage/files/{jsonable_encoder(path)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 501:
                raise NotImplementedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 502:
                raise BadGatewayError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CreateNotificationRequest,
    CreateNotificationResponse,
    GetNotificationResponse,
    Notification,
    NotificationId,
    NotificationStatus,
)

__all__ = [
    "CreateNotificationRequest",
    "CreateNotificationResponse",
    "GetNotificationResponse",
    "Notification",
    "NotificationId",
    "NotificationStatus",
]



# This file was auto-generated by Fern from our API Definition.

from ....v_1_3_0.vulnerabilityfinding.classes.types.vulnerability_finding import (
    VulnerabilityFinding as ocsf_v_1_3_0_vulnerabilityfinding_classes_types_vulnerability_finding_VulnerabilityFinding,
)

VulnerabilityFinding = ocsf_v_1_3_0_vulnerabilityfinding_classes_types_vulnerability_finding_VulnerabilityFinding



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .edge import Edge
from .node import Node
from .graph_query_language_id import GraphQueryLanguageId
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Graph(UncheckedBaseModel):
    """
    A graph data structure representation with nodes and edges.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph description - provides additional details about the graph's purpose and contents.
    """

    edges: typing.Optional[typing.List[Edge]] = pydantic.Field(default=None)
    """
    The edges/connections between nodes in the graph - contains the collection of <code>edge</code> objects defining relationships between nodes.
    """

    is_directed: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the graph is directed (<code>true</code>) or undirected (<code>false</code>).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph name - a human readable identifier for the graph.
    """

    nodes: typing.List[Node] = pydantic.Field()
    """
    The nodes/vertices of the graph - contains the collection of <code>node</code> objects that make up the graph.
    """

    query_language: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph query language, normalized to the caption of the <code>query_language_id</code> value.
    """

    query_language_id: typing.Optional[GraphQueryLanguageId] = pydantic.Field(
        default=None
    )
    """
    The normalized identifier of a graph query language that can be used to interact with the graph.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The graph type. Typically useful to represent the specifc type of graph that is used.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier of the graph - a unique ID to reference this specific graph.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .webhook_event import WebhookEvent
from .webhook_filter import WebhookFilter
from .webhook_id import WebhookId

__all__ = ["WebhookEvent", "WebhookFilter", "WebhookId"]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .key_value_object import KeyValueObject
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Service(UncheckedBaseModel):
    """
    The Service object describes characteristics of a service, <code> e.g. AWS EC2. </code>
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels associated with the service.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the service.
    """

    tags: typing.Optional[typing.List[KeyValueObject]] = pydantic.Field(default=None)
    """
    The list of tags; <code>{key:value}</code> pairs associated to the service.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the service.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the service.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..operation_base.types.async_operation_request_id import AsyncOperationRequestId
from ..core.request_options import RequestOptions
from .types.get_operation_response import GetOperationResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..operation_base.types.operation_input import OperationInput
from .types.create_operation_response import CreateOperationResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OperationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        operation_id: AsyncOperationRequestId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOperationResponse:
        """
        Returns the `Asynchronous Operation` object matching `{operationId}`.

        Parameters
        ----------
        operation_id : AsyncOperationRequestId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOperationResponse

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.operations.get(
            operation_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/operations/{jsonable_encoder(operation_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOperationResponse,
                    construct_type(
                        type_=GetOperationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        operation: str,
        input: OperationInput,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOperationResponse:
        """
        Creates an `Asynchronous Operation` object.

        Parameters
        ----------
        operation : str
            Name of the operation that will be run for this operation.

        input : OperationInput
            Input parameters to the operation that will be run for this operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOperationResponse

        Examples
        --------
        from synqly import SynqlyEngine
        from synqly.operation_base import OperationInput

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.operations.create(
            operation="string",
            input=OperationInput(
                filters=["string"],
                limit=1,
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/operations",
            method="POST",
            json={
                "operation": operation,
                "input": input,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateOperationResponse,
                    construct_type(
                        type_=CreateOperationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self,
        operation_id: AsyncOperationRequestId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Cancels the `Asynchronous Operation` matching `{operationId}`.

        Parameters
        ----------
        operation_id : AsyncOperationRequestId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from synqly import SynqlyEngine

        client = SynqlyEngine(
            token="YOUR_TOKEN",
        )
        client.operations.cancel(
            operation_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/operations/{jsonable_encoder(operation_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOperationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        operation_id: AsyncOperationRequestId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetOperationResponse:
        """
        Returns the `Asynchronous Operation` object matching `{operationId}`.

        Parameters
        ----------
        operation_id : AsyncOperationRequestId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOperationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.operations.get(
                operation_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/operations/{jsonable_encoder(operation_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOperationResponse,
                    construct_type(
                        type_=GetOperationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        operation: str,
        input: OperationInput,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOperationResponse:
        """
        Creates an `Asynchronous Operation` object.

        Parameters
        ----------
        operation : str
            Name of the operation that will be run for this operation.

        input : OperationInput
            Input parameters to the operation that will be run for this operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOperationResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine
        from synqly.operation_base import OperationInput

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.operations.create(
                operation="string",
                input=OperationInput(
                    filters=["string"],
                    limit=1,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/operations",
            method="POST",
            json={
                "operation": operation,
                "input": input,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    CreateOperationResponse,
                    construct_type(
                        type_=CreateOperationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self,
        operation_id: AsyncOperationRequestId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Cancels the `Asynchronous Operation` matching `{operationId}`.

        Parameters
        ----------
        operation_id : AsyncOperationRequestId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyEngine

        client = AsyncSynqlyEngine(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.operations.cancel(
                operation_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/operations/{jsonable_encoder(operation_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class IdentityClassOv(str, enum.Enum):
    INDIVIDUAL = "individual"
    GROUP = "group"
    SYSTEM = "system"
    ORGANIZATION = "organization"
    CLASS = "class"
    UNKNOWN = "unknown"
    _UNKNOWN = "__IDENTITYCLASSOV_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "IdentityClassOv":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        individual: typing.Callable[[], T_Result],
        group: typing.Callable[[], T_Result],
        system: typing.Callable[[], T_Result],
        organization: typing.Callable[[], T_Result],
        class_: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is IdentityClassOv.INDIVIDUAL:
            return individual()
        if self is IdentityClassOv.GROUP:
            return group()
        if self is IdentityClassOv.SYSTEM:
            return system()
        if self is IdentityClassOv.ORGANIZATION:
            return organization()
        if self is IdentityClassOv.CLASS:
            return class_()
        if self is IdentityClassOv.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .aws_provider_credential import AwsProviderCredential
from .aws_region import AwsRegion
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CloudSecurityAws(UncheckedBaseModel):
    """
    Configuration for the AWS Cloud Security Provider
    """

    credential: AwsProviderCredential
    region: AwsRegion = pydantic.Field()
    """
    The [AWS region](https://docs.aws.amazon.com/global-infrastructure/latest/regions/aws-regions.html) to use for the AWS Cloud Security Provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .account import Account
import pydantic
from .organization import Organization
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Cloud(UncheckedBaseModel):
    """
    The Cloud object contains information about a cloud account such as AWS Account ID, regions, etc.
    """

    account: typing.Optional[Account] = pydantic.Field(default=None)
    """
    The account object describes details about the account that was the source or target of the activity.
    """

    org: typing.Optional[Organization] = pydantic.Field(default=None)
    """
    Organization and org unit relevant to the event or object.
    """

    project_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of a Cloud project.
    """

    provider: str = pydantic.Field()
    """
    The unique name of the Cloud services provider, such as AWS, MS Azure, GCP, etc.
    """

    region: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the cloud region, as defined by the cloud provider.
    """

    zone: typing.Optional[str] = pydantic.Field(default=None)
    """
    The availability zone in the cloud region, as defined by the cloud provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.timestamp import Timestamp
import pydantic
import datetime as dt
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Epss(UncheckedBaseModel):
    """
    The Exploit Prediction Scoring System (EPSS) object describes the estimated probability a vulnerability will be exploited. EPSS is a community-driven effort to combine descriptive information about vulnerabilities (CVEs) with evidence of actual exploitation in-the-wild. (<a target='_blank' href='https://www.first.org/epss/'>EPSS</a>).
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The timestamp indicating when the EPSS score was calculated.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp indicating when the EPSS score was calculated.
    """

    percentile: typing.Optional[float] = pydantic.Field(default=None)
    """
    The EPSS score's percentile representing relative importance and ranking of the score in the larger EPSS dataset.
    """

    score: str = pydantic.Field()
    """
    The EPSS score representing the probability [0-1] of exploitation in the wild in the next 30 days (following score publication).
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the EPSS model used to calculate the score.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...capabilities_base.types.category_id import CategoryId
import pydantic
import typing
from ...capabilities_base.types.provider_id import ProviderId
from .provider_capabilities_base import ProviderCapabilitiesBase
from .connector_release import ConnectorRelease
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class Connector(UncheckedBaseModel):
    """
    Provides details of the Connector.
    """

    id: CategoryId = pydantic.Field()
    """
    Unique identifier for the Connector.
    """

    name: CategoryId = pydantic.Field()
    """
    Name of the connector.
    """

    fullname: str = pydantic.Field()
    """
    Display name of the Connector.
    """

    description: str = pydantic.Field()
    """
    Description of the Connector.
    """

    provider_ids: typing.List[ProviderId] = pydantic.Field()
    """
    List of Providers that implement the Connector.
    """

    providers: typing.Optional[typing.List[ProviderCapabilitiesBase]] = pydantic.Field(
        default=None
    )
    """
    List of capabilities for Providers that implement the Connector.
    Uses ProviderCapabilitiesBase which omits the expandable connector
    field since nested providers never have expanded connectors.
    """

    connector: typing.Optional[CategoryId] = pydantic.Field(default=None)
    """
    DEPRECATED – use `id` instead.
    """

    release: typing.Optional[ConnectorRelease] = pydantic.Field(default=None)
    """
    Connector availability information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .data_security_category_id import DataSecurityCategoryId
from .data_security_confidentiality_id import DataSecurityConfidentialityId
from .data_security_data_lifecycle_state_id import DataSecurityDataLifecycleStateId
from .data_security_detection_system_id import DataSecurityDetectionSystemId
from .policy import Policy
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class DataSecurity(UncheckedBaseModel):
    """
    The Data Security object describes the characteristics, techniques and content of a Data Loss Prevention (DLP), Data Loss Detection (DLD), Data Classification, or similar tools' finding, alert, or detection mechanism(s).
    """

    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the data classification category that data matched into, e.g. Financial, Personal, Governmental, etc.
    """

    category_id: typing.Optional[DataSecurityCategoryId] = pydantic.Field(default=None)
    """
    The normalized identifier of the data classification category.
    """

    confidentiality: typing.Optional[str] = pydantic.Field(default=None)
    """
    The file content confidentiality, normalized to the confidentiality_id value. In the case of 'Other', it is defined by the event source.
    """

    confidentiality_id: typing.Optional[DataSecurityConfidentialityId] = pydantic.Field(
        default=None
    )
    """
    The normalized identifier of the file content confidentiality indicator.
    """

    data_lifecycle_state: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the stage or state that the data was in. E.g., Data-at-Rest, Data-in-Transit, etc.
    """

    data_lifecycle_state_id: typing.Optional[DataSecurityDataLifecycleStateId] = (
        pydantic.Field(default=None)
    )
    """
    The stage or state that the data was in when it was assessed or scanned by a data security tool.
    """

    detection_pattern: typing.Optional[str] = pydantic.Field(default=None)
    """
    Specific pattern, algorithm, fingerpint, or model used for detection.
    """

    detection_system: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the type of data security tool or system that the finding, detection, or alert originated from. E.g., Endpoint, Secure Email Gateway, etc.
    """

    detection_system_id: typing.Optional[DataSecurityDetectionSystemId] = (
        pydantic.Field(default=None)
    )
    """
    The type of data security tool or system that the finding, detection, or alert originated from.
    """

    pattern_match: typing.Optional[str] = pydantic.Field(default=None)
    """
    A text, binary, file name, or datastore that matched against a detection rule.
    """

    policy: typing.Optional[Policy] = pydantic.Field(default=None)
    """
    Details about the policy that triggered the finding.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class WebResource(UncheckedBaseModel):
    """
    The Web Resource object describes characteristics of a web resource that was affected by the activity/event.
    """

    data: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(default=None)
    """
    Details of the web resource, e.g, <code>file</code> details, <code>search</code> results or application-defined resource.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the web resource.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The list of labels/tags associated to a resource.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the web resource.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The web resource type as defined by the event source.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the web resource.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL pointing towards the source of the web resource.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from ...capabilities_base.types.provider_id import ProviderId
import pydantic
from ...capabilities_base.types.category_id import CategoryId
import typing
from .provider_operations import ProviderOperations
from .provider_release import ProviderRelease
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ProviderCapabilitiesBase(UncheckedBaseModel):
    """
    Base properties for provider capabilities. This type is used both
    directly (when providers are nested in connectors) and extended by
    ProviderCapabilities (which adds the expandable connector field).
    """

    id: ProviderId = pydantic.Field()
    """
    Unique identifier for the Provider.
    """

    name: str = pydantic.Field()
    """
    Name of the Provider.
    """

    fullname: str = pydantic.Field()
    """
    Display name of the Provider.
    """

    vendor: str = pydantic.Field()
    """
    Display name of the Provider vendor.
    """

    description: str = pydantic.Field()
    """
    Description of the Provider.
    """

    connector_id: CategoryId = pydantic.Field()
    """
    Id of the Connector that the Provider implements.
    """

    operations: typing.Optional[typing.List[ProviderOperations]] = pydantic.Field(
        default=None
    )
    """
    Operations that the Provider implements.
    """

    provider_config: typing.Optional[typing.Optional[typing.Any]] = pydantic.Field(
        default=None
    )
    """
    Details on the specific configuration options for the Provider.
    """

    release: typing.Optional[ProviderRelease] = pydantic.Field(default=None)
    """
    Provider availability information.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import ActivityId, BaseEvent, CategoryUid, ClassUid, SeverityId, StatusId

__all__ = [
    "ActivityId",
    "BaseEvent",
    "CategoryUid",
    "ClassUid",
    "SeverityId",
    "StatusId",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class RemoteFieldScope(str, enum.Enum):
    STANDARD = "standard"
    INTEGRATION_CUSTOM = "integration_custom"
    PROJECT_CUSTOM = "project_custom"
    TICKET_CUSTOM = "ticket_custom"
    UNKNOWN = "unknown"
    _UNKNOWN = "__REMOTEFIELDSCOPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "RemoteFieldScope":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        standard: typing.Callable[[], T_Result],
        integration_custom: typing.Callable[[], T_Result],
        project_custom: typing.Callable[[], T_Result],
        ticket_custom: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is RemoteFieldScope.STANDARD:
            return standard()
        if self is RemoteFieldScope.INTEGRATION_CUSTOM:
            return integration_custom()
        if self is RemoteFieldScope.PROJECT_CUSTOM:
            return project_custom()
        if self is RemoteFieldScope.TICKET_CUSTOM:
            return ticket_custom()
        if self is RemoteFieldScope.UNKNOWN:
            return unknown()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .feature import Feature
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Product(UncheckedBaseModel):
    """
    The Product object describes characteristics of a software product.
    """

    cpe_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Common Platform Enumeration (CPE) name as described by (<a target='_blank' href='https://nvd.nist.gov/products/cpe'>NIST</a>) For example: <code>cpe:/a:apple:safari:16.2</code>.
    """

    distribution_mode: typing.Optional[str] = pydantic.Field(default=None)
    """
    Indicates the source from which the app was installed, such as the app store.
    """

    feature: typing.Optional[Feature] = pydantic.Field(default=None)
    """
    The feature that reported the event.
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The two letter lower case language codes, as defined by <a target='_blank' href='https://en.wikipedia.org/wiki/ISO_639-1'>ISO 639-1</a>. For example: <code>en</code> (English), <code>de</code> (German), or <code>fr</code> (French).
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the product.
    """

    path: typing.Optional[str] = pydantic.Field(default=None)
    """
    The installation path of the product.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the product.
    """

    url_string: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The URL pointing towards the product.
    """

    vendor_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the vendor of the product.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of the product, as defined by the event source. For example: <code>2013.1.3-beta</code>.
    """

    workload: typing.Optional[str] = pydantic.Field(default=None)
    """
    The product workload associated with the event.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .remote_field_schema import RemoteFieldSchema
from .remote_field_scope import RemoteFieldScope
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RemoteField(UncheckedBaseModel):
    field_id: str = pydantic.Field()
    """
    Remote ID of the field.
    """

    schema_: typing.Optional[RemoteFieldSchema] = pydantic.Field(
        alias="schema", default=None
    )
    """
    Schema of the field.
    """

    field_scope: RemoteFieldScope = pydantic.Field()
    """
    Scope of remote field - standard or some kind of customization?
    """

    provider_field_name: str = pydantic.Field()
    """
    Name of field in the remote ticketing provider.
    """

    provider_field_path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Path to the field in the remote ticketing provider. Uses dot notation for nested fields.
    """

    project_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Project id to which this field is scoped to
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Cwe(UncheckedBaseModel):
    """
    The CWE object represents a weakness in a software system that can be exploited by a threat actor to perform an attack. The CWE object is based on the <a target='_blank' href='https://cwe.mitre.org/'>Common Weakness Enumeration (CWE)</a> catalog.
    """

    caption: typing.Optional[str] = pydantic.Field(default=None)
    """
    The caption assigned to the Common Weakness Enumeration unique identifier.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    URL pointing to the CWE Specification. For more information see <a target='_blank' href='https://cwe.mitre.org/'>CWE.</a>
    """

    uid: str = pydantic.Field()
    """
    The Common Weakness Enumeration unique number assigned to a specific weakness. A CWE Identifier begins "CWE" followed by a sequence of digits that acts as a unique identifier. For example: <code>CWE-123</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .incident_response_escalation_policy import IncidentResponseEscalationPolicy
from .incident_response_escalation_rule import IncidentResponseEscalationRule
from .incident_response_escalation_target_union import (
    IncidentResponseEscalationTargetUnion,
    IncidentResponseEscalationTargetUnion_IncidentResponseSchedule,
    IncidentResponseEscalationTargetUnion_IncidentResponseScheduleReference,
    IncidentResponseEscalationTargetUnion_IncidentResponseTeam,
    IncidentResponseEscalationTargetUnion_IncidentResponseTeamReference,
    IncidentResponseEscalationTargetUnion_IncidentResponseUser,
    IncidentResponseEscalationTargetUnion_IncidentResponseUserReference,
)
from .incident_response_query_escalation_policies_response import (
    IncidentResponseQueryEscalationPoliciesResponse,
)
from .incident_response_query_escalation_policy_users_on_call_response import (
    IncidentResponseQueryEscalationPolicyUsersOnCallResponse,
)
from .incident_response_schedule import IncidentResponseSchedule
from .incident_response_schedule_reference import IncidentResponseScheduleReference
from .incident_response_team import IncidentResponseTeam
from .incident_response_team_reference import IncidentResponseTeamReference
from .incident_response_team_union import (
    IncidentResponseTeamUnion,
    IncidentResponseTeamUnion_IncidentResponseTeam,
    IncidentResponseTeamUnion_IncidentResponseTeamReference,
)
from .incident_response_user import IncidentResponseUser
from .incident_response_user_reference import IncidentResponseUserReference
from .incident_response_user_union import (
    IncidentResponseUserUnion,
    IncidentResponseUserUnion_IncidentResponseUser,
    IncidentResponseUserUnion_IncidentResponseUserReference,
)

__all__ = [
    "IncidentResponseEscalationPolicy",
    "IncidentResponseEscalationRule",
    "IncidentResponseEscalationTargetUnion",
    "IncidentResponseEscalationTargetUnion_IncidentResponseSchedule",
    "IncidentResponseEscalationTargetUnion_IncidentResponseScheduleReference",
    "IncidentResponseEscalationTargetUnion_IncidentResponseTeam",
    "IncidentResponseEscalationTargetUnion_IncidentResponseTeamReference",
    "IncidentResponseEscalationTargetUnion_IncidentResponseUser",
    "IncidentResponseEscalationTargetUnion_IncidentResponseUserReference",
    "IncidentResponseQueryEscalationPoliciesResponse",
    "IncidentResponseQueryEscalationPolicyUsersOnCallResponse",
    "IncidentResponseSchedule",
    "IncidentResponseScheduleReference",
    "IncidentResponseTeam",
    "IncidentResponseTeamReference",
    "IncidentResponseTeamUnion",
    "IncidentResponseTeamUnion_IncidentResponseTeam",
    "IncidentResponseTeamUnion_IncidentResponseTeamReference",
    "IncidentResponseUser",
    "IncidentResponseUserReference",
    "IncidentResponseUserUnion",
    "IncidentResponseUserUnion_IncidentResponseUser",
    "IncidentResponseUserUnion_IncidentResponseUserReference",
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class CisCsc(UncheckedBaseModel):
    """
    The CIS Critical Security Control (CSC) contains information as defined by the Center for Internet Security Critical Security Control <a target='_blank' href='https://www.cisecurity.org/controls'>(CIS CSC)</a>. Prioritized set of actions to protect your organization and data from cyber-attack vectors.
    """

    control: str = pydantic.Field()
    """
    A Control is prescriptive, prioritized, and simplified set of best practices that one can use to strengthen their cybersecurity posture. e.g. AWS SecurityHub Controls, CIS Controls.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The CIS critical security control version.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .scheduled_operation_id import ScheduledOperationId
import pydantic
from .operation_frequency import OperationFrequency
import typing
from .operation_incremental_pull_config import OperationIncrementalPullConfig
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class OperationSchedule(UncheckedBaseModel):
    operation: ScheduledOperationId = pydantic.Field()
    """
    ID of the operation to schedule (e.g., query_alerts, query_endpoints).
    """

    enabled: bool = pydantic.Field()
    """
    Whether the schedule is active. When false, no executions will occur.
    """

    frequency: OperationFrequency = pydantic.Field()
    """
    How often the operation should execute (e.g., every 4 hours).
    If an execution takes longer than the interval, the next execution
    will start immediately after the previous one completes.
    """

    filters: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Optional filters to apply when querying data from the source API.
    Format and available filters depend on the specific operation.
    Example: "severity[eq]critical" or "status[eq]active".
    """

    backfill_from_time: typing.Optional[str] = pydantic.Field(default=None)
    """
    Starting point for the initial data collection. No data before this time will be fetched.
    Accepts:
    
    - RFC 3339 datetime: "2024-01-01T00:00:00Z"
    - Relative duration: "7d" (7 days ago), "24h" (24 hours ago), "30m" (30 minutes ago)
    - "max": Collect all available historical data
      If not specified, starts from the operation's creation time.
    """

    incremental_config: typing.Optional[OperationIncrementalPullConfig] = (
        pydantic.Field(default=None)
    )
    """
    Configuration for how data is fetched on each execution.
    By default, operations fetch only data that changed since the last run (incremental)
    and the time filter used depends on the operation.
    Use this to override the default behavior or switch to full snapshots.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .azure_blob_credential import AzureBlobCredential
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class SinkAzureBlob(UncheckedBaseModel):
    """
    Configuration for Azure Blob Storage as a Sink provider. Events are written directly to an Azure Blob Storage container in compressed JSON format.
    """

    bucket: str = pydantic.Field()
    """
    Name of the Azure Blob Storage container
    """

    credential: AzureBlobCredential = pydantic.Field()
    """
    Azure token with write access to the configured blob container.
    """

    path: str = pydantic.Field()
    """
    Files will be written under this path.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from ...base.types.timestamp import Timestamp
import pydantic
import datetime as dt
from .group import Group
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Table(UncheckedBaseModel):
    """
    The table object represents a table within a structured relational database or datastore, which contains columns and rows of data that are able to be create, updated, deleted and queried.
    """

    created_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The time when the table was known to have been created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The time when the table was known to have been created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the table.
    """

    groups: typing.Optional[typing.List[Group]] = pydantic.Field(default=None)
    """
    The group names to which the table belongs.
    """

    modified_time: typing.Optional[Timestamp] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the table.
    """

    modified_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The most recent time when any changes, updates, or modifications were made within the table.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The table name, ordinarily as assigned by a database administrator.
    """

    size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of the data table in bytes.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique identifier of the table.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ....v_1_5_0.applicationsecurityposturefinding.classes.types.application_security_posture_finding import (
    ApplicationSecurityPostureFinding as ocsf_v_1_5_0_applicationsecurityposturefinding_classes_types_application_security_posture_finding_ApplicationSecurityPostureFinding,
)

ApplicationSecurityPostureFinding = ocsf_v_1_5_0_applicationsecurityposturefinding_classes_types_application_security_posture_finding_ApplicationSecurityPostureFinding



# This file was auto-generated by Fern from our API Definition.

from ...engine.types.api_query_response import ApiQueryResponse
import typing
from .project import Project
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ListProjectsResponse(ApiQueryResponse):
    result: typing.List[Project]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import pydantic
from .open_search_credential import OpenSearchCredential
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SinkOpenSearch(UncheckedBaseModel):
    """
    Configuration for OpenSearch search and analytics engine. Supports both managed and self-hosted OpenSearch deployments
    """

    create_index: str = pydantic.Field()
    """
    The index or data stream to use when writing events.
    """

    credential: OpenSearchCredential
    skip_tls_verify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, skips verification of the OpenSearch TLS certificate.
    """

    url: str = pydantic.Field()
    """
    Base URL for the OpenSearch API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .types import (
    QueryCloudResourceInventoryResponse,
    QueryCloudResourceInventoryResponseGeneric,
    QueryCloudSecurityThreatsResponse,
    QueryCloudSecurityThreatsResponseGeneric,
    QueryComplianceFindingsResponse,
    QueryComplianceFindingsResponseGeneric,
    QueryEventsResponse,
    QueryEventsResponseGeneric,
    QueryIomsResponse,
    QueryIomsResponseGeneric,
)

__all__ = [
    "QueryCloudResourceInventoryResponse",
    "QueryCloudResourceInventoryResponseGeneric",
    "QueryCloudSecurityThreatsResponse",
    "QueryCloudSecurityThreatsResponseGeneric",
    "QueryComplianceFindingsResponse",
    "QueryComplianceFindingsResponseGeneric",
    "QueryEventsResponse",
    "QueryEventsResponseGeneric",
    "QueryIomsResponse",
    "QueryIomsResponseGeneric",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .attachment_id import AttachmentId
import pydantic
from .ticket_id import TicketId
import typing
import datetime as dt
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class AttachmentMetadata(UncheckedBaseModel):
    """
    Attachment in a ticketing system
    """

    id: AttachmentId = pydantic.Field()
    """
    Unique identifier for this attachment.
    """

    ticket_id: TicketId = pydantic.Field()
    """
    The ticket this attachment is associated with.
    """

    file_name: str = pydantic.Field()
    """
    The name of the file.
    """

    file_type: str = pydantic.Field()
    """
    The type of the file.
    """

    file_size: typing.Optional[int] = pydantic.Field(default=None)
    """
    The size of the file in bytes.
    """

    created_date: dt.datetime = pydantic.Field()
    """
    The date the attachment was created.
    """

    creator: str = pydantic.Field()
    """
    The user who created the attachment.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the attachment metadata schema. The names and values of those attributes are specific to the provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .connection_state import ConnectionState
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class NetworkQuarantineRequest(UncheckedBaseModel):
    state: ConnectionState = pydantic.Field()
    """
    The connection state (Connect or Disconnect) to enforce for the provided endpoint IDs.
    """

    endpoint_ids: typing.List[str] = pydantic.Field()
    """
    The list of endpoint IDs to enforce the connection state on.
    """

    comment: typing.Optional[str] = pydantic.Field(default=None)
    """
    A comment to include with the quarantine action.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .defender_credential import DefenderCredential
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class EdrDefender(UncheckedBaseModel):
    """
    Configuration for Microsoft Defender for Endpoint.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/defender-setup)
    """

    credential: DefenderCredential
    tenant_id: str = pydantic.Field()
    """
    Tenant ID for the Microsoft Defender Management Console.
    """

    url: str = pydantic.Field()
    """
    Base URL for the Microsoft Defender API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
